﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace ADMHSample
{
    public partial class DevForm : Form
    {
        private short DevId; //Device Id

        private int SmplMaxLeng; //最大取り込み長
        private BinaryWriter[] SaveFile = new BinaryWriter[2]; // 保存ファイルストリーム
        private int[] SavedDataNum = new int[2];
        //サンプリング条件----------------------------------
        private byte ClkSrc; //クロック源設定 0:200MHz 1:20.48MHz 2:16.384MHz 3:12.8MHz 4:1.92MHz 5:Ext
        private byte ClkDivN; //クロック分周率
        private byte SmplAve; //サンプリング平均化設定 2^n

        private byte TrgSel; //トリガ選択 0:外部 1:立ち上がり 2:立下り 3:ソフトウェア
        private int Threshold; //アナログトリガ閾値
        private int NoiseCancel; //ノイズキャンセルレベル
        public DevForm()
        {
            InitializeComponent();
        }

        // 自然数かどうか判定する関数
        private Boolean IsNatural(string strNum)
        {
            if (System.Text.RegularExpressions.Regex.IsMatch(strNum, "^[1-9]\\d{0,8}$")) // Charactor check [0-9]
                return true;
            else
                return false;
        }

        // 自然数かどうか判定する関数(0を含む)
        private Boolean IsNatural0(string strNum)
        {
            if (System.Text.RegularExpressions.Regex.IsMatch(strNum, "^\\d{1,9}$")) // Charactor check [0-9]
                return true;
            else
                return false;
        }

        // DevFormの初期化を行う
        public void SetId(short Id)
        {
            short retErr;//return error
            byte IOData;
            byte Type1;
            byte Type2;

            DevId = Id;

            this.Text = "ID = " + DevId.ToString() + "  Panel";
            retErr = TUSBADMH.Tusbadmh_Dio_Read(DevId, out IOData); //Read digital I/O
            if (retErr != 0)
            {
                MessageBox.Show(TUSBADMH.GetErrMessage(retErr));
                this.Hide();
                return;
            }
            else
            {
                //Output state
                DOChk3.Checked = ((IOData & 0x80) != 0) ? true : false;
                DOChk2.Checked = ((IOData & 0x40) != 0) ? true : false;
                DOChk1.Checked = ((IOData & 0x20) != 0) ? true : false;
                DOChk0.Checked = ((IOData & 0x10) != 0) ? true : false;
                //Input State
                DIDisp3.Text = ((IOData & 0x8) != 0) ? "High" : "Low";
                DIDisp3.Text = ((IOData & 0x8) != 0) ? "High" : "Low";
                DIDisp3.Text = ((IOData & 0x8) != 0) ? "High" : "Low";
                DIDisp3.Text = ((IOData & 0x8) != 0) ? "High" : "Low";
            }

            DivideN.Text = "2";
            Check_ClockSetting();

            retErr = TUSBADMH.Tusbadmh_CheckInputType(DevId, out Type1, out Type2);//Read input range
            if (retErr != 0)
            {
                MessageBox.Show(TUSBADMH.GetErrMessage(retErr));
                this.Hide();
                return;
            }
            else
            {
                if (Type1 != 0)
                    Ch1InputSel1.Checked = true;
                else
                    Ch1InputSel2.Checked = true;

                if (Type2 != 0)
                    Ch2InputSel1.Checked = true;
                else
                    Ch2InputSel2.Checked = true;
            }
            retErr = TUSBADMH.Tusbadmh_TransLimit(DevId, 50000);
            if (retErr != 0)
            {
                MessageBox.Show(TUSBADMH.GetErrMessage(retErr));
            }

            TrgSel4.Checked = true;//Select soft trigger
            TrgSet();
            ThLevel.Text = "32768"; //Select center of range
            NCLevel.Text = "1638"; //50mV
            SmplMode2.Checked = true;//Continuation mode
            CycleLeng.Text = "1000"; //Sampling length per one trigger. (1～1,048,576)
            PreTrgLeng.Text = "0"; //Pre trigger length. (0～1,048,576)
            TotalLeng.Text = "100000"; //Total sampling length.  (1～2,000,000,000)
            ClockSrc.SelectedIndex = 0; //200MHz
            DivideN.Text = "8"; //divide 8 (200MHz / 8 -> 25MHz )
            AverageN.SelectedIndex = 0; //Average 1

            Timer1.Interval = 100;
            Timer1.Enabled = true;
        }

        private void DevForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (Timer2.Enabled)//タイマー2が動作中であれば
            {
                Timer2.Enabled = false;
                StartStop();//停止
            }
            Timer1.Enabled = false;//タイマー1を停止します
        }

        //ディジタル出力設定
        private void DO_Set()
        {
            short retErr;
            byte OutData;

            OutData = 0;

            if (DOChk0.Checked)//ビット0
                OutData += 0x01;
            if (DOChk1.Checked)//ビット1
                OutData += 0x02;
            if (DOChk2.Checked)//ビット2
                OutData += 0x04;
            if (DOChk3.Checked)//ビット3
                OutData += 0x08;

            retErr = TUSBADMH.Tusbadmh_Dio_Write(DevId, OutData);//設定
            if (retErr != 0)
            {
                MessageBox.Show(TUSBADMH.GetErrMessage(retErr));
                this.Hide();
                return;
            }
        }

        private void DOChk_CheckedChanged(object sender, EventArgs e)
        {
            DO_Set();
            DOChk0.Text = (DOChk0.Checked) ? "High" : "Low";
            DOChk1.Text = (DOChk1.Checked) ? "High" : "Low";
            DOChk2.Text = (DOChk2.Checked) ? "High" : "Low";
            DOChk3.Text = (DOChk3.Checked) ? "High" : "Low";
        }

        
        //クロック設定条件の確認
        void Check_ClockSetting()
        {
            double tmpClk;
            double tmpDivN;
            int tmpInt;

            if( ClockSrc.SelectedIndex < 0 )
            {
                return;
            }

            ClkSrc = (byte) ClockSrc.SelectedIndex;

            if( ClkSrc == 5 )
            {
                IntClockFreq.Text = "";
                return;
            }

            switch(ClkSrc)
            {
                case 0://200MHz
                    tmpClk = 200.0;
                    break;
                case 1://20.48MHz
                    tmpClk = 20.48;
                    break;
                case 2://16.384MHz
                    tmpClk = 16.384;
                    break;
                case 3://12.8MHz
                    tmpClk = 12.8;
                    break;
                case 4://1.92MHz
                    tmpClk = 1.92;
                    break;
                default:
                    tmpClk = 1;
                    break;
            }

            if( IsNatural((DivideN.Text) ) )//分周率入力が数値か
            {
                int.TryParse(DivideN.Text, out tmpInt);//分周率を得る
                if (tmpInt != 0)//分周率が0でないこと
                {
                    tmpClk /= tmpInt;
                    if( tmpClk > 25.0 || tmpClk < 1.0 )//クロックは1～25MHzでなければならない
                    {
                        IntClockFreq.Text = "Out of Range";
                        return;
                    }
                    ClkDivN = (byte) tmpInt;
               }
                else
                {
                    IntClockFreq.Text = "Invalid value";
                    return;
                }
            }
            else
            {
                IntClockFreq.Text = "Invalid value";
                return;
            }

            if( AverageN.SelectedIndex < 0 )
                return;

            SmplAve = (byte) AverageN.SelectedIndex;
            double.TryParse(AverageN.Text,out tmpDivN);
            tmpClk = tmpClk / tmpDivN;
            if( tmpClk >= 1.0 )
                IntClockFreq.Text = tmpClk.ToString("0.0000") + "MHz";
            else
                IntClockFreq.Text = (tmpClk * 1000.0).ToString("0.0000") + "kHz";
            
        }

        private void ClockSrc_TextChanged(object sender, EventArgs e)
        {
            Check_ClockSetting();
        }

        private void ClockSrc_SelectedIndexChanged(object sender, EventArgs e)
        {
            Check_ClockSetting();
        }

        private void DivideN_TextChanged(object sender, EventArgs e)
        {
            Check_ClockSetting();
        }

        //入力レンジ選択
        private void SetInputType()
        {
            short retErr;
            byte Type1;
            byte Type2;

            Type1 = (byte)((Ch1InputSel1.Checked == true) ? 1 : 0); //ch1 0:unipolar 1:bipolar
            Type2 = (byte)((Ch2InputSel1.Checked == true) ? 1 : 0); //ch2 0:unipolar 1:bipolar

            retErr = TUSBADMH.Tusbadmh_InputType(DevId, Type1, Type2); //Set input range
            if (retErr != 0)
            {
                MessageBox.Show(TUSBADMH.GetErrMessage(retErr));
                this.Hide();
                return;
            }
        }

        private void AverageN_TextChanged(object sender, EventArgs e)
        {
            Check_ClockSetting();
        }

        private void AverageN_SelectedIndexChanged(object sender, EventArgs e)
        {
            Check_ClockSetting();
        }

        private void Ch1InputSel_CheckedChanged(object sender, EventArgs e)
        {
            SetInputType();
            ThLevelChange();
        }

        private void Ch2InputSel_CheckedChanged(object sender, EventArgs e)
        {
            SetInputType();
        }

        //ディジタル入力チェック
        private void DI_Check()
        {
            byte IOData;
            short retErr;

            retErr = TUSBADMH.Tusbadmh_Dio_Read(DevId, out IOData);
            if (retErr != 0)
            {
                MessageBox.Show(TUSBADMH.GetErrMessage(retErr));
                this.Hide();
                return;
            }
            else
            {
                //入力状態
                DIDisp3.Text = ((IOData & 0x08) != 0) ? "High" : "Low";//ビット3
                DIDisp2.Text = ((IOData & 0x04) != 0) ? "High" : "Low";//ビット2
                DIDisp1.Text = ((IOData & 0x02) != 0) ? "High" : "Low";//ビット1
                DIDisp0.Text = ((IOData & 0x01) != 0) ? "High" : "Low";//ビット0
            }
        }

        private void FileConvBtn_Click(object sender, EventArgs e)
        {
            string ReadFileName;
            string WriteFileName;
            BinaryReader ReadFile;//読み出しファイルストリーム
            StreamWriter WriteFile;// 保存ファイルストリーム

            do
            {
                OpenFileDialog1.FileName = "";
                OpenFileDialog1.Title = "バイナリファイル指定";
                OpenFileDialog1.Filter = "バイナリファイル (*.dat)|*.dat";
                if (OpenFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
                {
                    return;
                }
            } while (OpenFileDialog1.FileName == "");
            ReadFileName = OpenFileDialog1.FileName;

            do
            {
                SaveFileDialog1.OverwritePrompt = true;
                SaveFileDialog1.FileName = "";
                SaveFileDialog1.Title = "変換先ファイル指定";
                SaveFileDialog1.Filter = "テキスト保存ファイル (*.csv)|*.csv";

                if (SaveFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
                    return;
            } while (SaveFileDialog1.FileName == "");
            WriteFileName = SaveFileDialog1.FileName;

            ReadFile = new BinaryReader(File.OpenRead(ReadFileName)); //ファイルを開く
            if (File.Exists(SaveFileDialog1.FileName)) // 既存ファイルチェック
                File.Delete(SaveFileDialog1.FileName);
            WriteFile = new StreamWriter(File.OpenWrite(WriteFileName)); // ファイルを開く

            string[] ClkSelStr = { "200MHz", "20.48MHz", "16.384MHz", "12.8MHz", "1.92MHz", "External" };
            string[] ImpRngStr = { "Unipolar", "Bipolar" };
            string[] TrgSelStr = { "External", "Up edge", "Down edge", "Software" };
            string[] SmpModStr = { "Continuation", "Repeat" };

            WriteFile.WriteLine("Start date time" + "," + System.DateTime.FromBinary(ReadFile.ReadInt64()).ToString()); //日時
            WriteFile.WriteLine("Channel" + "," + ReadFile.ReadByte().ToString()); //ch
            WriteFile.WriteLine("Clock select" + "," + ClkSelStr[ReadFile.ReadByte()]); //クロック源
            WriteFile.WriteLine("Dividing rate" + "," + ReadFile.ReadByte().ToString()); //クロック分周率
            WriteFile.WriteLine("Average Cnt" + "," + (2 ^ ReadFile.ReadByte()).ToString()); //サンプリング平均化乗数
            WriteFile.WriteLine("Input range" + "," + ImpRngStr[ReadFile.ReadByte()]); //レンジ 0:ユニポーラ　1:バイポーラ
            WriteFile.WriteLine("Trigger select" + "," + TrgSelStr[ReadFile.ReadByte()]); //トリガ選択 0:外部 1:立ち上がり 2:立下り 3:ソフトウェア
            WriteFile.WriteLine("Trigger threshold" + "," + ReadFile.ReadInt32().ToString()); //アナログトリガ閾値
            WriteFile.WriteLine("Noise cancel level" + "," + ReadFile.ReadInt32().ToString()); //ノイズキャンセルレベル
            WriteFile.WriteLine("Sampling mode" + "," + SmpModStr[ReadFile.ReadByte()]); //取り込み動作モード 0:連続 1:リピート
            WriteFile.WriteLine("Sampling length per one trigger" + "," + ReadFile.ReadInt32().ToString()); //1トリガ当たりの取り込み長
            WriteFile.WriteLine("Pre trigger length" + "," + ReadFile.ReadInt32().ToString()); //プレトリガ長

            try
            {
                do
                {
                    WriteFile.WriteLine(ReadFile.ReadInt32() & 0xFFFF);
                } while (true);
            }
            catch (Exception ex)//ファイルエンド
            {
                if (!(ex is EndOfStreamException))
                {
                    MessageBox.Show(ex.ToString());
                }
            }

            ReadFile.Close();
            WriteFile.Close();

            MessageBox.Show("変換が終了しました");
        }

        //ノイズキャンセルレベル設定値のチェック
        private void NCLevel_TextChanged(object sender, EventArgs e)
        {
            int Level;
            double Volt;

            if (IsNatural0(NCLevel.Text))//自然数であるか
            {
                int.TryParse(NCLevel.Text, out Level);
                if (Level >= 0 && Level <= 3277)//範囲内であるか
                {
                    Volt = 2.0 * Level / 65536.0 * 1000.0;
                    NCLevelV.Text = Volt.ToString("0.0") + "[mV]";
                }
                else
                    NCLevelV.Text = "";
            }
            else
                NCLevelV.Text = "";
        }

        private void SmplMode_CheckedChanged(object sender, EventArgs e)
        {
            if (SmplMode1.Checked)//リピートモードが選択された
            {
                CycleLeng.Enabled = true;
                PreTrgLeng.Enabled = false;
            }
            else//連続モードが選択された
            {
                CycleLeng.Enabled = false;
                PreTrgLeng.Enabled = true;
            }
        }

        //クロック設定値のチェックと設定
        private Boolean ClockCheckAndSet()
        {
            short retErr;
            double tmpClk;
            int tmpInt;

            ClkSrc = (byte) ClockSrc.SelectedIndex;

            if( ClkSrc != 5 )//内部クロック
            {
                switch( ClkSrc)
                {

                    case 0://200MHz
                        tmpClk = 200.0;
                        break;
                    case 1://20.48MHz
                        tmpClk = 20.48;
                        break;
                    case 2://16.384MHz
                        tmpClk = 16.384;
                        break;
                    case 3://12.8MHz
                        tmpClk = 12.8;
                        break;
                    case 4://1.92MHz
                        tmpClk = 1.92;
                        break;
                    default:
                        tmpClk = 1;
                        break;
                }

                if( IsNatural(DivideN.Text))//クロック分周率が自然数がどうか
                {
                    int.TryParse(DivideN.Text,out tmpInt);
                    if (tmpInt != 0)//分周率が0でないこと
                    {
                        tmpClk /= tmpInt;
                        if( tmpClk > 25.0 || tmpClk < 1.0 )//クロックは1～25MHzでなければならない
                        {
                            MessageBox.Show("クロック設定範囲は(1～25MHz)です");
                            return false;
                        }
                        ClkDivN = (byte) tmpInt;
                    }
                    else
                    {
                        MessageBox.Show("分周率が不正です");
                        return false;
                    }
                }
                else
                {
                    MessageBox.Show("クロック入力の形式が不正です");
                    return false;
                }
            }
            else//外部クロック
            {
                if( IsNatural(DivideN.Text) )//クロック分周率が自然数がどうか
                {
                    int.TryParse(DivideN.Text, out tmpInt);
                    if (tmpInt < 1 || tmpInt > 25)
                    {
                        MessageBox.Show("分周比が不正です");
                        return false;
                    }
                    ClkDivN = (byte)tmpInt;
                }
                else
                {
                    MessageBox.Show("クロック入力の形式が不正です");
                    return false;
                }
            }

            SmplAve = (byte) AverageN.SelectedIndex;

            retErr = TUSBADMH.Tusbadmh_Clock_Select(DevId, ClkSrc, (byte)(ClkDivN - 1), SmplAve); //クロック設定

            if( retErr != 0 )
            {
                MessageBox.Show(TUSBADMH.GetErrMessage(retErr));
                this.Hide();
                return false;
            }
            return true;
        }

        //トリガ条件チェックと設定
        private Boolean TriggerCheckAndSet()
        {
            short retErr;

            if (TrgSel2.Checked || TrgSel3.Checked)//アナログトリガの場合
            {
                if (IsNatural(ThLevel.Text))//トリガレベルが自然数であるか
                {
                    int.TryParse(ThLevel.Text, out Threshold);
                    if (Threshold < 1 || Threshold > 65536)//範囲の確認
                    {
                        MessageBox.Show("アナログトリガレベル設定範囲が正しくありません");
                        return false;
                    }
                }
                else
                {
                    MessageBox.Show("アナログトリガレベル設定値が不正です");
                    return false;
                }

                if (IsNatural0(NCLevel.Text))//ノイズキャンセルレベルが自然数であるか
                {
                    int.TryParse(NCLevel.Text, out NoiseCancel);
                    if (NoiseCancel < 0 || NoiseCancel > 3277)//範囲の確認
                    {
                        MessageBox.Show("ノイズキャンセル設定範囲が正しくありません");
                        return false;
                    }
                }
                else
                {
                    MessageBox.Show("ノイズキャンセル設定値が不正です");
                    return false;
                }

                retErr = TUSBADMH.Tusbadmh_ThLevel_Set(DevId, Threshold, NoiseCancel);//スレッショルドレベル設定
                if (retErr != 0)
                {
                    MessageBox.Show(TUSBADMH.GetErrMessage(retErr));
                    this.Hide();
                    return false;
                }
            }
            return true;
        }

        //取り込み設定情報書き込み
        private void SaveSamplingSettings(int CycLen, int PreLen, byte Mode)
        {
            System.DateTime startTime;//日時
            byte Type1;//ch1 0:ユニポーラ 1:バイポーラ
            byte Type2;//ch2 0:ユニポーラ 1:バイポーラ
            byte InCh;//入力チャンネル
            startTime = System.DateTime.Now;
            Type1 = (byte)((Ch1InputSel1.Checked) ? 1 : 0);
            Type2 = (byte)((Ch1InputSel2.Checked) ? 1 : 0);

            InCh = 1;//チャンネル1
            SaveFile[0].Write(startTime.ToBinary()); //現在日時
            SaveFile[0].Write(InCh); //チャンネル1
            SaveFile[0].Write(ClkSrc); //クロック源
            SaveFile[0].Write(ClkDivN); //内部クロック分周率
            SaveFile[0].Write(SmplAve); //サンプリング平均化
            SaveFile[0].Write(Type1); //ch1 0:ユニポーラ　1:バイポーラ
            SaveFile[0].Write(TrgSel); //トリガ選択 0:外部 1:立ち上がり 2:立下り 3:ソフトウェア
            SaveFile[0].Write(Threshold); //アナログトリガ閾値
            SaveFile[0].Write(NoiseCancel); //ノイズキャンセルレベル
            SaveFile[0].Write(Mode); //取り込み動作モード 0:連続 1:リピート
            SaveFile[0].Write(CycLen); //1トリガ当たりの取り込み長
            SaveFile[0].Write(PreLen); //プレトリガ長
            if (Ch1Only.Checked == false)
            {
                InCh = 2; //チャンネル2
                SaveFile[1].Write(startTime.ToBinary()); //現在日時
                SaveFile[1].Write(InCh); //チャンネル2
                SaveFile[1].Write(ClkSrc); //クロック源
                SaveFile[1].Write(ClkDivN); //内部クロック分周率
                SaveFile[1].Write(SmplAve); //サンプリング平均化
                SaveFile[1].Write(Type2); //ch2 0:ユニポーラ　1:バイポーラ
                SaveFile[1].Write(TrgSel); //トリガ選択 0:外部 1:立ち上がり 2:立下り 3:ソフトウェア
                SaveFile[1].Write(Threshold); //アナログトリガ閾値
                SaveFile[1].Write(NoiseCancel); //ノイズキャンセルレベル
                SaveFile[1].Write(Mode); //取り込み動作モード 0:連続 1:リピート
                SaveFile[1].Write(CycLen); //1トリガ当たりの取り込み長
                SaveFile[1].Write(PreLen); //プレトリガ長
            }
        }

        //連続変換開始
        private Boolean SamplingStart()
        {
            short retErr;
            int CycLen;
            int PreLen;
            byte Mode;
            string[] FileName = new string[2];

            //クロック設定チェック
            if (ClockCheckAndSet() == false)
                return false;

            //トリガ設定チェック
            if (TriggerCheckAndSet() == false)
                return false;

            //取り込み数チェック
            if (SmplMode1.Checked)//リピートモード
            {
                if (IsNatural(CycleLeng.Text))
                {
                    int.TryParse(CycleLeng.Text, out CycLen);
                    if (CycLen < 1 || CycLen > 1048576)
                    {
                        MessageBox.Show("1トリガ当たりの取り込み長が範囲外です");
                        return false;
                    }
                }
                else
                {
                    MessageBox.Show("1トリガ当たりの取り込み長入力が不正です");
                    return false;
                }
                PreLen = 0;
                Mode = 1;
            }
            else//連続モード
            {
                if (IsNatural0(PreTrgLeng.Text))
                {
                    int.TryParse(PreTrgLeng.Text, out PreLen);
                    if (PreLen < 0 || PreLen > 1048576)
                    {
                        MessageBox.Show("プレトリガ長が範囲外です");
                        return false;
                    }
                }
                else
                {
                    MessageBox.Show("プレトリガ長入力が不正です");
                    return false;
                }
                CycLen = 1;
                Mode = 0;
            }
            if (IsNatural(TotalLeng.Text))
            {
                int.TryParse(TotalLeng.Text, out SmplMaxLeng);
                if (SmplMaxLeng < 1 || SmplMaxLeng > 2000000000)
                {
                    MessageBox.Show("最大取り込み長が範囲外です");
                    return false;
                }
            }
            else
            {
                MessageBox.Show("最大取り込み長入力が不正です");
                return false;
            }

            //トリガ選択
            if (TrgSel1.Checked)
                TrgSel = 0;
            else if (TrgSel2.Checked)
                TrgSel = 1;
            else if (TrgSel3.Checked)
                TrgSel = 2;
            else
                TrgSel = 3;

            //保存ファイルオープン
            do
            {
                SaveFileDialog1.FileName = "";
                SaveFileDialog1.Title = "ADM Ch1 保存ファイル指定";
                SaveFileDialog1.Filter = "バイナリ保存ファイル (*.dat)|*.dat";
                SaveFileDialog1.CheckFileExists = false;

                if (SaveFileDialog1.ShowDialog() == DialogResult.Cancel)
                    return false;
            } while (SaveFileDialog1.FileName == "");
            FileName[0] = SaveFileDialog1.FileName;
            if (File.Exists(FileName[0]))//既存ファイルチェック
            {
                File.Delete(FileName[0]);
            }
            SaveFile[0] = new BinaryWriter(File.OpenWrite(FileName[0]));//ファイルを開く

            if (Ch1Only.Checked == false)
            {
                do
                {
                    SaveFileDialog1.FileName = "";
                    SaveFileDialog1.Title = "ADM Ch2 保存ファイル指定";
                    SaveFileDialog1.Filter = "バイナリ保存ファイル (*.dat)|*.dat";
                    SaveFileDialog1.CheckFileExists = false;

                    if (SaveFileDialog1.ShowDialog() == DialogResult.Cancel)
                        return false;
                } while (SaveFileDialog1.FileName == "" || SaveFileDialog1.FileName == FileName[0]);
                FileName[1] = SaveFileDialog1.FileName;
                if (File.Exists(FileName[1]))//既存ファイルチェック
                    File.Delete(FileName[1]);
                SaveFile[1] = new BinaryWriter(File.OpenWrite(FileName[1]));//ファイルを開く
            }

            //取り込み設定情報書き込み
            SaveSamplingSettings(CycLen, PreLen, Mode);

            //変換開始
            byte Ch1OnlyFlg;
            Ch1OnlyFlg = (byte)((Ch1Only.Checked) ? 1 : 0);
            retErr = TUSBADMH.Tusbadmh_Adc_Start(DevId, CycLen, PreLen, TrgSel, Mode, Ch1OnlyFlg);//変換開始
            if (retErr != 0)
            {
                MessageBox.Show(TUSBADMH.GetErrMessage(retErr));
                this.Hide();
                return false;
            }

            SavedDataNum[0] = 0;
            SavedDataNum[1] = 0;
            Timer2.Interval = 200;
            Timer2.Enabled = true;
            return true;
        }

        //変換停止
        private Boolean SamplingStop()
        {
            short retErr;
            //変換終了
            Timer2.Enabled = false;

            retErr = TUSBADMH.Tusbadmh_Adc_Stop(DevId);//停止

            SaveFile[0].Close();
            SaveFile[1].Close();

            if (retErr != 0)
            {
                MessageBox.Show(TUSBADMH.GetErrMessage(retErr));
                this.Hide();
                return false;
            }
            return true;
        }

        //パネル上のコントロールの有効、無効の設定
        private void EnableDisableControl(Boolean Enable)
        {
            ClockSrc.Enabled = Enable;
            AverageN.Enabled = Enable;
            DivideN.Enabled = Enable;
            Ch1InputSel1.Enabled = Enable;
            Ch1InputSel2.Enabled = Enable;
            Ch2InputSel1.Enabled = Enable;
            Ch2InputSel2.Enabled = Enable;
            TrgSel1.Enabled = Enable;
            TrgSel2.Enabled = Enable;
            TrgSel3.Enabled = Enable;
            TrgSel4.Enabled = Enable;
            ThLevel.Enabled = Enable;
            NCLevel.Enabled = Enable;
            SmplMode1.Enabled = Enable;
            SmplMode2.Enabled = Enable;
            TotalLeng.Enabled = Enable;
            FileConvBtn.Enabled = Enable;
            Ch1Only.Enabled = Enable;

            if (SmplMode1.Checked)//繰り返しモード
            {
                CycleLeng.Enabled = Enable;
                PreTrgLeng.Enabled = false;
            }
            else//連続モード
            {
                CycleLeng.Enabled = false;
                PreTrgLeng.Enabled = Enable;
            }
        }

        // 変換開始、停止ボタン押下
        private void StartStop()
        {
            if (StartStopBtn.Text == "Start" )//開始の場合
            {
                if (SamplingStart())
                {
                    EnableDisableControl(false);
                    StartStopBtn.Text = "Stop";
                }
            }
            else//停止の場合
            {
                if (SamplingStop())
                {
                    EnableDisableControl(true);
                    StartStopBtn.Text = "Start";
                    StatusText.Text = "";
                }
            }
        }

        private void StartStopBtn_Click(object sender, EventArgs e)
        {
            StartStop();
        }

        //トリガスレッショルドレベルが変わった時の処理
        private void ThLevelChange()
        {
            int Level;
            double Volt;

            if( IsNatural(ThLevel.Text))
            {
                int.TryParse(ThLevel.Text,out Level);
                if( Level >= 1 && Level <= 65534 )
                {
                    if( Ch1InputSel1.Checked )//ユニポーラ 0-2V
                        Volt = 2.0 * Level / 65536.0;
                    else
                        Volt = 2.0 * Level / 65536.0 - 1.0;
                    ThLevelV.Text = Volt.ToString("0.000") + "[V]";
                }
                else
                {
                    ThLevelV.Text = "";
                }
            }
            else
            {
                ThLevelV.Text = "";
            }
        }

        private void ThLevel_TextChanged(object sender, EventArgs e)
        {
            ThLevelChange();
        }

        // タイマー1処理
        private void Timer1_Tick(object sender, EventArgs e)
        {
            DI_Check();
        }

        // タイマー2の処理
        //変換時の監視
        private void Timer2_Tick(object sender, EventArgs e)
        {
            byte Status;
            byte OvfSt;
            int[] Leng = new int[2];
            int[] UsableRate = new int[2];
            int[] Data;
            short ret;
            int cnt;
            
            //ステータスチェック
            if (TUSBADMH.Tusbadmh_Status_Get(DevId, out Status, out OvfSt) == 0)
            {
                switch (Status)
                {
                    case 0://停止
                        StatusText.Text = "";
                        break;
                    case 1://トリガ前
                        StatusText.Text = "Trigger waiting.";
                        break;
                    case 2://トリガ後
                        switch (OvfSt)
                        {
                            case 1://ch1オーバーフロー
                                StatusText.Text = "Ch1 buffer overflow.";
                                break;
                            case 2://ch2オーバーフロー
                                StatusText.Text = "Ch2 buffer overflow.";
                                break;
                            case 3://ch1とch2オーバーフロー
                                StatusText.Text = "Ch1 && Ch2 buffer overflow.";
                                break;
                            default:
                                StatusText.Text = "Sampling";
                                break;
                        }
                        break;
                }
            }

            //データ取り込み
            if( TUSBADMH.Tusbadmh_Length(DevId,out Leng[0],out Leng[1],out UsableRate[0],out UsableRate[1]) == 0 )
            {
                if( ( SavedDataNum[0] + Leng[0] ) > SmplMaxLeng )// ch1データが多すぎる場合
                {
                    Leng[0] = SmplMaxLeng - SavedDataNum[0];
                }
                if( Leng[0] != 0)
                {
                    Data = new int[Leng[0]];

                    ret = TUSBADMH.Tusbadmh_Data_Get(DevId,0,Data,ref Leng[0]);
                    if( ret == 0 )
                    {
                        for( cnt = 0; cnt < Leng[0] ; cnt++ )
                        {
                            SaveFile[0].Write(Data[cnt]);
                        }
                    }
                }
                SavedDataNum[0] += Leng[0];

                if( Ch1Only.Checked == false )
                {
                    if( (SavedDataNum[1] + Leng[1] ) > SmplMaxLeng )//ch2データが多すぎる場合
                    {
                        Leng[1] = SmplMaxLeng - SavedDataNum[1];
                    }
                    if( Leng[1] != 0 )
                    {
                        Data = new int[Leng[1]];

                        ret = TUSBADMH.Tusbadmh_Data_Get(DevId, 1, Data, ref Leng[1] );
                        if( ret == 0 )
                        {
                            for( cnt = 0 ; cnt < Leng[1] ; cnt++ )
                            {
                                SaveFile[1].Write(Data[cnt]);
                            }
                        }
                    }
                    SavedDataNum[1] += Leng[1];
                }

                Ch1SampledLeng.Text = SavedDataNum[0].ToString();
                Ch1BufRate.Text = UsableRate[0].ToString("000") + "%";
                Ch2SampledLeng.Text = SavedDataNum[1].ToString();
                Ch2BufRate.Text = UsableRate[1].ToString("000") + "%";

                if( Ch1Only.Checked ) 
                {
                    if( SavedDataNum[0] >= SmplMaxLeng )
                    {
                        StartStop();//停止
                    }
                }
                else
                {
                    if( (SavedDataNum[0] >= SmplMaxLeng) && ( SavedDataNum[1] >= SmplMaxLeng ) )
                    {
                        StartStop();//停止
                    }
                }
            }
        }
        private void TrgSet()
        {
            ThLevel.Enabled = !(TrgSel1.Checked || TrgSel4.Checked);
            NCLevel.Enabled = !(TrgSel1.Checked || TrgSel4.Checked);
            TriggerBtn.Enabled = TrgSel4.Checked;
        }

        private void TrgSel_CheckedChanged(object sender, EventArgs e)
        {
            TrgSet();
        }

        private void TriggerBtn_Click(object sender, EventArgs e)
        {
            short retErr;
            retErr = TUSBADMH.Tusbadmh_Trigger(DevId);
            if (retErr != 0)
            {
                MessageBox.Show(TUSBADMH.GetErrMessage(retErr));
                this.Hide();
                return;
            }
        }
    }
}
