﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace ADMHSample
{
    public partial class MainForm : Form
    {
        private Boolean[] DevStatus = new Boolean[16];// Device Connection Flag TRUE : connecting
        private DevForm[] IdDevForm = new DevForm[16];// Device Control Panel Form Object
        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            short DevId;

            Timer1.Interval = 300;
            Timer1.Enabled = true;

            DevIdSel.SelectedIndex = 0;

            // Initialize
            for (DevId = 0; DevId < 16; DevId++)
            {
                DevStatus[DevId] = false; // Device is not Connected
            }
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            short DevId;

            Timer1.Enabled = false;
            for (DevId = 0; DevId < 16; DevId++)
            {
                if (IdDevForm[DevId] != null)
                {
                    if (IdDevForm[DevId].Visible)
                    {
                        IdDevForm[DevId].Close();
                        DevStatus[DevId] = false;
                        TUSBADMH.Tusbadmh_Device_Close(DevId);
                    }
                }
            }
        }

        private void FileConvBtn_Click(object sender, EventArgs e)
        {
            string ReadFileName;
            string WriteFileName;
            BinaryReader ReadFile;//読み出しファイルストリーム
            StreamWriter WriteFile;// 保存ファイルストリーム

            do
            {
                OpenFileDialog1.FileName = "";
                OpenFileDialog1.Title = "バイナリファイル指定";
                OpenFileDialog1.Filter = "バイナリファイル (*.dat)|*.dat";
                if (OpenFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
                {
                    return;
                }
            } while (OpenFileDialog1.FileName == "");
            ReadFileName = OpenFileDialog1.FileName;

            do
            {
                SaveFileDialog1.OverwritePrompt = true;
                SaveFileDialog1.FileName = "";
                SaveFileDialog1.Title = "変換先ファイル指定";
                SaveFileDialog1.Filter = "テキスト保存ファイル (*.csv)|*.csv";

                if (SaveFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
                    return;
            } while (SaveFileDialog1.FileName == "");
            WriteFileName = SaveFileDialog1.FileName;

            ReadFile = new BinaryReader(File.OpenRead(ReadFileName)); //ファイルを開く
            if (File.Exists(SaveFileDialog1.FileName)) // 既存ファイルチェック
                File.Delete(SaveFileDialog1.FileName);
            WriteFile = new StreamWriter(File.OpenWrite(WriteFileName)); // ファイルを開く

            string[] ClkSelStr = { "200MHz", "20.48MHz", "16.384MHz", "12.8MHz", "1.92MHz", "External" };
            string[] ImpRngStr = { "Unipolar", "Bipolar" };
            string[] TrgSelStr = { "External", "Up edge", "Down edge", "Software" };
            string[] SmpModStr = { "Continuation", "Repeat" };

            WriteFile.WriteLine("Start date time" + "," + System.DateTime.FromBinary(ReadFile.ReadInt64()).ToString()); //日時
            WriteFile.WriteLine("Channel" + "," + ReadFile.ReadByte().ToString()); //ch
            WriteFile.WriteLine("Clock select" + "," + ClkSelStr[ReadFile.ReadByte()]); //クロック源
            WriteFile.WriteLine("Dividing rate" + "," + ReadFile.ReadByte().ToString()); //クロック分周率
            WriteFile.WriteLine("Average Cnt" + "," + (2 ^ ReadFile.ReadByte()).ToString()); //サンプリング平均化乗数
            WriteFile.WriteLine("Input range" + "," + ImpRngStr[ReadFile.ReadByte()]); //レンジ 0:ユニポーラ　1:バイポーラ
            WriteFile.WriteLine("Trigger select" + "," + TrgSelStr[ReadFile.ReadByte()]); //トリガ選択 0:外部 1:立ち上がり 2:立下り 3:ソフトウェア
            WriteFile.WriteLine("Trigger threshold" + "," + ReadFile.ReadInt32().ToString()); //アナログトリガ閾値
            WriteFile.WriteLine("Noise cancel level" + "," + ReadFile.ReadInt32().ToString()); //ノイズキャンセルレベル
            WriteFile.WriteLine("Sampling mode" + "," + SmpModStr[ReadFile.ReadByte()]); //取り込み動作モード 0:連続 1:リピート
            WriteFile.WriteLine("Sampling length per one trigger" + "," + ReadFile.ReadInt32().ToString()); //1トリガ当たりの取り込み長
            WriteFile.WriteLine("Pre trigger length" + "," + ReadFile.ReadInt32().ToString()); //プレトリガ長

            try
            {
                do
                {
                    WriteFile.WriteLine(ReadFile.ReadInt32() & 0xFFFF);
                } while (true);
            }
            catch (Exception ex)//ファイルエンド
            {
                if (!(ex is EndOfStreamException))
                {
                    MessageBox.Show(ex.ToString());
                }
            }

            ReadFile.Close();
            WriteFile.Close();

            MessageBox.Show("変換が終了しました");
        }

        // Device Control Panel Open
        private void OpenPanel(short Id)
        {
            DevStatus[Id] = true;
            IdDevForm[Id] = new DevForm();// Device Control Panel Form Create
            IdDevForm[Id].SetId(Id);
            IdDevForm[Id].Show();// Form Show
        }

        private void OpenBtn_Click(object sender, EventArgs e)
        {
            short DevId;

            DevId = (short)DevIdSel.SelectedIndex;
            if (DevStatus[DevId] == false)// Device is not connected
            {
                if (TUSBADMH.Tusbadmh_Device_Open(DevId) == 0)// Connection check
                    OpenPanel(DevId);
            }
        }

        private void Timer1_Tick(object sender, EventArgs e)
        {
            short DevId;

            for (DevId = 0; DevId < 16; DevId++)
            {
                if (DevStatus[DevId])
                {
                    if (IdDevForm[DevId].Visible == false) // Device is not visible
                    {
                        TUSBADMH.Tusbadmh_Device_Close(DevId);
                        DevStatus[DevId] = false;
                    }
                }
            }
        }

        private void EndAppBtn_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}
